from typing import List


def main(args: List[str]) -> bool:
    print('Usage:')
    print('vsw <command> [options]')
    print('')
    print('')
    print('Commands:')
    print('setup              Start your local agent service to talk with repo.')
    print('init               Initialize schema, connection for the first time.')
    print('publish            Publish a software credential to the vsw-repo')
    print('verify             Verify if a software credential is correct')
    print('list               List of various useful info')
    print('exit               This will terminate aca-py agent')
    print('')
    print('')
    print('General Options:')
    print('-h, --help         Show help')


import json

import requests

import vsw

from vsw.commands import init
from vsw.commands import list
from tests import test_publish

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()


def test_do_schema():
    init.do_schema("software-certificate")


def test_connection_repo():
    # create connection with repo
    init.connection_repo()


def test_list_schema():
    list.get_schema(vsw_config)
    list.get_credential_definition(vsw_config)


def test_remove_all_connection():
    schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections'
    response = requests.get(schema_url)
    results = json.loads(response.text)["results"]
    for result in results:
        schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections/{result["connection_id"]}/remove'
        requests.post(schema_url)
        print(f"Remove connection id: {result['connection_id']}")

    repo_schema_url = f'{vsw_repo_config.get("host")}/connections'
    response = requests.get(repo_schema_url)
    results = json.loads(response.text)["results"]
    for result in results:
        vsw_repo_url = f'{vsw_repo_config.get("host")}/connections/{result["connection_id"]}/remove'
        requests.post(vsw_repo_url)
        print(f"Remove repo connection id: {result['connection_id']}")


# clean all history data
def test_clean_history_data():
    test_remove_all_connection()
    test_publish.test_clean_all_records()


import json
import requests

from vsw.commands import publish
import vsw

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
local_conn_id = "45c40084-edcb-42e9-ab39-6b7bf9c0cb2f"
repo_conn_id = "569ee27c-17b3-4629-ad40-ef2a4c6fb653"
credential_definition_id = "CqdPVBFnDs6uUzrrd8Prmw:3:CL:2950:default"
cred_ex_id = "151c7bfb-a672-4507-99eb-aabd9bb3a494"


def test_parse_args():
    publish.parse_args()


def test_publish():
    publish.send_proposal(repo_conn_id, "CqdPVBFnDs6uUzrrd8Prmw", "Felix", "1.0", "12121212",
                          "http://images.pccoo.cn/bar/2012426/20124261343081s.jpg",
                  "", "")

# 1 Repo Holder sends a proposal to the issuer (issuer receives proposal)
def test_send_proposal():
    print('#1 Repo Holder sends a proposal to the issuer (issuer receives proposal)')
    vsw_repo_url = f'{repo_url_host}/issue-credential/send-proposal'
    res = requests.post(vsw_repo_url, json={
        # "schema_id": "CqdPVBFnDs6uUzrrd8Prmw:2:software-certificate:1.0",  # TODO
        "comment": "Felix Debug Test",
        # "schema_issuer_did": "WgWxqztrNooG92RXvxSTWv",  # TODO
        "auto_remove": False,
        "trace": True,
        # "cred_def_id": "CqdPVBFnDs6uUzrrd8Prmw:3:CL:2918:default",  # TODO
        "connection_id": repo_conn_id,
        # "schema_name": "preferences",
        # "schema_version": "1.0",
        "credential_proposal": {
            "@type": "did:sov:CqdPVBFnDs6uUzrrd8Prmw;spec/issue-credential/1.0/credential-preview",
            "attributes": [{
                "name": "developer-did",
                "value": "M4yVd8vfSE7qo9PeRG5ArM"  # TODO
            },
                {
                    "name": "software-version",
                    "value": "1.0"
                },
                {
                    "name": "software-name",
                    "value": "HappyBirds"
                },
                {
                    "name": "software-did",
                    "value": "80da584735d94d5948bc5b450e6fdb837afae78d2a27682d095e9b9b576cf95e"
                },
                {
                    "name": "url",
                    "value": "http://images.pccoo.cn/bar/2012426/20124261343081s.jpg"
                },
                {
                    "name": "alt-url1",
                    "value": ""
                },
                {
                    "name": "alt-url2",
                    "value": ""
                },
                {
                    "name": "hash",
                    "value": "digest"
                },
                {
                    "name": "alt-hash",
                    "value": "digest"
                }
            ]
        },
        # "issuer_did": "WgWxqztrNooG92RXvxSTWv"  # TODO
    })
    print(json.loads(res.text))


# 2 Issuer sends an offer to the holder based on the proposal (holder receives offer)
def test_send_offer():
    print('#2 Issuer sends an offer to the holder based on the proposal (holder receives offer)')
    url = f'{vsw_url_host}/issue-credential/send-offer'
    res = requests.post(url, json={
        "comment": "Felix Test",
        "auto_remove": True,
        "auto_issue": True,
        "trace": False,
        "cred_def_id": credential_definition_id,
        "connection_id": local_conn_id,
        "credential_preview": {
            "@type": "did:sov:BzCbsNYhMrjHiqZDTUASHg;spec/issue-credential/1.0/credential-preview",
            "attributes": [{
                "name": "developer-did",
                "value": "M4yVd8vfSE7qo9PeRG5ArM"  # TODO
            },
                {
                    "name": "software-version",
                    "value": "1.0"
                },
                {
                    "name": "software-name",
                    "value": "Felix Test"
                },
                {
                    "name": "software-did",
                    "value": "80da584735d94d5948bc5b450e6fdb837afae78d2a27682d095e9b9b576cf95e"
                }
            ]
        }
    })
    print(json.loads(res.text))


# 3 Issuer sends an offer to the holder based on the proposal (holder receives offer)
def test_send_offer_cred_ex_id():
    print('#3 Issuer sends an offer to the holder based on the proposal (holder receives offer)')
    url = f'{vsw_url_host}/issue-credential/records/{cred_ex_id}/send-proposal'
    res = requests.post(url)
    print(json.loads(res.text))


# 4 Repo-Holder sends a request to the issuer (issuer receives request)
def test_send_request():
    print('#4 Repo-Holder sends a request to the issuer (issuer receives request)')
    url = f'{repo_url_host}/issue-credential/records/{cred_ex_id}/send-request'
    res = requests.post(url)
    print(json.loads(res.text))


# 5 Issuer sends credential to holder (holder receives credentials)
def test_issue(cred_ex_id):
    print('#5 Issuer sends credential to holder (holder receives credentials)')
    url = f'{vsw_url_host}/issue-credential/records/{cred_ex_id}/issue'
    res = requests.post(url, json={
        "comment": "Felix Test"
    })
    print(json.loads(res.text))


# 6 Repo Holder stores credential (holder sends acknowledge to issuer)
def test_store(cred_ex_id, credential_id):
    print('#6 Repo Holder stores credential (holder sends acknowledge to issuer)')
    url = f'{repo_url_host}/issue-credential/records/{cred_ex_id}/store'
    res = requests.post(url, json={
        "credential_id": credential_id
    })
    print(json.loads(res.text))


# automatically
def create():
    url = f'{vsw_url_host}/issue-credential/create'
    res = requests.post(url, json={
        "cred_def_id": "WgWxqztrNooG92RXvxSTWv:3:CL:20:tag",
        "schema_issuer_did": "WgWxqztrNooG92RXvxSTWv",
        "auto_remove": True,
        "comment": "string",
        "issuer_did": "WgWxqztrNooG92RXvxSTWv",
        "credential_proposal": {
            "@type": "did:sov:BzCbsNYhMrjHiqZDTUASHg;spec/issue-credential/1.0/credential-preview",
            "attributes": [{
                "name": "developer-did",
                "value": "M4yVd8vfSE7qo9PeRG5ArM"  # TODO
            },
                {
                    "name": "software-version",
                    "value": "1.0"
                },
                {
                    "name": "software-name",
                    "value": "Felix Test"
                },
                {
                    "name": "software-did",
                    "value": "80da584735d94d5948bc5b450e6fdb837afae78d2a27682d095e9b9b576cf95e"
                }
            ]
        },
        "schema_id": "WgWxqztrNooG92RXvxSTWv:2:schema_name:1.0",
        "schema_name": "preferences",
        "trace": False,
        "schema_version": "1.0"
    })
    print(res)


# automatically
def test_send():
    url = f'{vsw_url_host}/issue-credential/send'
    res = requests.post(url, json={
        "cred_def_id": "NVAbHh9DuKxFA8URWVp7Rb:3:CL:2911:default",
        "schema_issuer_did": "NVAbHh9DuKxFA8URWVp7Rb",  # TODO
        "auto_remove": True,
        "comment": "Felix Test",
        "issuer_did": "NVAbHh9DuKxFA8URWVp7Rb",  # TODO
        "credential_proposal": {
            "@type": "did:sov:BzCbsNYhMrjHiqZDTUASHg;spec/issue-credential/1.0/credential-preview",
            "attributes": [{
                "name": "developer-did",
                "value": "M4yVd8vfSE7qo9PeRG5ArM"  # TODO
            },
                {
                    "name": "software-version",
                    "value": "1.0"
                },
                {
                    "name": "software-name",
                    "value": "Felix Test"
                },
                {
                    "name": "software-did",
                    "value": "80da584735d94d5948bc5b450e6fdb837afae78d2a27682d095e9b9b576cf95e"
                }
            ]
        },
        "schema_id": "NVAbHh9DuKxFA8URWVp7Rb:2:software-certificate:1.0",
        "schema_name": "software-certificate",
        "trace": False,
        "connection_id": "08e4560d-89b0-4131-bf8b-a6e0dae9a62e",
        "schema_version": "1.0"
    })
    print(json.loads(res.text))


def test_get_public_did():
    publish.get_public_did()


def test_get_credentail_records():
    publish.get_credential_record("d073e5f4-b825-4cae-8d65-1ec62ba4d448")


def test_clean_all_records():
    local = f'{vsw_url_host}/issue-credential/records'
    response = requests.get(local)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            credential_exchange_id = result["credential_exchange_id"]
            url = f'{vsw_url_host}/issue-credential/records/{credential_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed credential records {credential_exchange_id}')

    local = f'{vsw_url_host}/present-proof/records'
    response = requests.get(local)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            presentation_exchange_id = result["presentation_exchange_id"]
            url = f'{vsw_url_host}/present-proof/records/{presentation_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed present proof records {presentation_exchange_id}')

    repo = f'{repo_url_host}/issue-credential/records'
    response = requests.get(repo)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            credential_exchange_id = result["credential_exchange_id"]
            url = f'{repo_url_host}/issue-credential/records/{credential_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed credential records {credential_exchange_id}')

    repo = f'{repo_url_host}/present-proof/records'
    response = requests.get(repo)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            presentation_exchange_id = result["presentation_exchange_id"]
            url = f'{repo_url_host}/present-proof/records/{presentation_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed present proof records {presentation_exchange_id}')

    # Remove Credential
    repo = f'{repo_url_host}/credentials?count={100}'
    response = requests.get(repo)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            referent = result["referent"]
            url = f'{repo_url_host}/credential/{referent}/remove'
            requests.post(url)
            print(f'Removed referent {referent}')



from vsw.commands import setup


def test_start_agent():
    setup.start_agent("default", "default")


def test_get_seed():
    setup.get_seed()


def test_provision():
    setup.provision("tr", "tr")


import argparse
from typing import Any, Dict, List

import pkg_resources

args = argparse.Namespace()


def _registered_commands(
        group: str = "vsw.registered_commands",
) -> Dict[str, pkg_resources.EntryPoint]:
    registered_commands = pkg_resources.iter_entry_points(group=group)
    return {c.name: c for c in registered_commands}


def dispatch(argv: List[str]) -> Any:
    registered_commands = _registered_commands()
    parser = argparse.ArgumentParser(prog="vsw")
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s version {}".format(pkg_resources.require("vsw")[0].version),
    )
    parser.add_argument(
        "command",
        choices=registered_commands.keys(),
    )
    parser.add_argument(
        "args",
        help=argparse.SUPPRESS,
        nargs=argparse.REMAINDER,
    )

    parser.parse_args(argv, namespace=args)

    main = registered_commands[args.command].load()

    return main(args.args)



import argparse
import json
import time
from typing import List
from urllib.parse import urljoin
import requests

import vsw.utils
from vsw.log import Log

logger = Log(__name__).logger
timeout = 60

def main(args: List[str]) -> bool:
    args = parse_args(args)
    try:
        if args.connection:
            connection_repo()
        if args.schema:
            do_schema(args.schema)
    except KeyboardInterrupt:
        print(" ==> Exit init")


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--schema", required=False, help="The schema name")
    parser.add_argument('-c', '--connection', action='store_true')
    return parser.parse_args(args)


def do_schema(schema):
    vsw_config = vsw.utils.get_vsw_agent()
    schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/schemas'
    response = requests.post(schema_url, json={
        "schema_version": "0.2",
        "attributes": ["developer-did","software-version","software-name","software-did", "hash",
                       "alt-hash", "url", "alt-url1", "alt-url2"],
        "schema_name": schema
    })
    schema_res = json.loads(response.text)
    schema_id = schema_res["schema_id"]
    logger.info(f'Created schema_id: {schema_id}')

    credential_definition_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/credential-definitions'
    res = requests.post(credential_definition_url, json={
        "revocation_registry_size": 0,
        "support_revocation": False,
        "schema_id": schema_id,
        "tag": "default"
    })
    credential_definition_res = json.loads(res.text)
    logger.info(f'Created credential_definition_id: {credential_definition_res["credential_definition_id"]}')


def connection_repo():
    try:
        vsw_config = vsw.utils.get_vsw_agent()
        vsw_repo_config = vsw.utils.get_repo_host()
        vsw_repo_url = f'{vsw_repo_config.get("host")}/connections/create-invitation?alias={vsw_repo_config.get("label")}&auto_accept=true'
        logger.info(f'Create invitation to: {vsw_repo_url}')
        response = requests.post(vsw_repo_url)
        res = json.loads(response.text)
        logger.info(res)

        local_url = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections/receive-invitation?alias={vsw_config.get("label")}'
        logger.info(f'Receive invitation {local_url}')
        # invitation = res["invitation"]
        # body = {
        #     "label": invitation["label"],
        #     "did": invitation["did"],
        #     "@type": invitation["@type"],
        #     "@id": invitation["@id"]
        # }
        invitation = res["invitation"]
        body = {
            "label": invitation["label"],
            "serviceEndpoint": invitation["serviceEndpoint"],
            "recipientKeys": invitation["recipientKeys"],
            "@id": invitation["@id"]
        }
        ss = requests.post(local_url, json=body)
        invitation_response = json.loads(ss.text)
        logger.info(invitation_response)
        connection_id = invitation_response["connection_id"]
        times = 0
        while times <= timeout:
            connection_res = get_connection(connection_id, vsw_config)
            logger.info(f'waiting state update, current state: {connection_res["state"]}')
            if connection_res["state"] == "active":
                logger.info('Created connection with Repo')
                break;
            else:
                times += 1;
        if times > timeout:
            logger.error("Sorry, there might be some issue during initializing connection.")

    except Exception as err:
        logger.error('connection vsw-repo failed', err)


def get_connection(connection_id, vsw_config):
    time.sleep(1)  # wait communicate complete automatically between agents
    url = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections/{connection_id}'
    connection_response = requests.get(url)
    res = json.loads(connection_response.text)
    return res


import argparse
import json
from typing import List
from uuid import uuid4
from urllib.parse import urljoin
import validators
import requests
import vsw.utils
import time
from vsw.log import Log

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    try:
        parser = argparse.ArgumentParser(prog="vsw verify")
        parser.add_argument("-s", "--software-name", required=True, help="The software name")
        parser.add_argument("-u", "--url", required=True, help="The software download url")
        parser.add_argument("-i", "--issuer-did", required=True, help="The issuer did")
        parsed_args = parser.parse_args(args)
        if not validators.url(parsed_args.url):
            print('The software package url is wrong, please check')
            return
        execute(parsed_args.software_name, parsed_args.issuer_did, parsed_args.url)
    except KeyboardInterrupt:
        print(" ==> Exit verify!")


def execute(software_name, issuer_did, download_url):
    connection = get_client_connection()
    logger.info("Executing verify, please wait for response...")
    credentials = check_credential(issuer_did, software_name, download_url)
    if len(credentials) == 0:
        logger.error("No found credential, please check if the specified conditions are correct.")
    else:
        proof_response = send_request(connection["connection_id"], software_name, issuer_did)
        presentation_exchange_id = proof_response["presentation_exchange_id"]
        logger.info(f'presentation_exchange_id: {presentation_exchange_id}')
        times = 0
        while times <= timeout:
            presentation_proof_result = retrieve_result(presentation_exchange_id)
            state = presentation_proof_result["state"]
            print(f"waiting state update, current state is: {state}")
            if state == "verified":
                pres_req = presentation_proof_result["presentation_request"]
                pres = presentation_proof_result["presentation"]
                is_proof_of_software_certificate = (
                        pres_req["name"] == "Proof of Software Certificate"
                )
                if is_proof_of_software_certificate:
                    # check claims
                    proof_dict = {}
                    for (referent, attr_spec) in pres_req["requested_attributes"].items():
                        proof_dict[attr_spec['name']] = pres['requested_proof']['revealed_attrs'][referent]['raw']
                        logger.info(
                            f"{attr_spec['name']}: "
                            f"{pres['requested_proof']['revealed_attrs'][referent]['raw']}"
                        )
                    credential_hash = proof_dict['hash']
                    url = proof_dict['url']
                    if credential_hash is None or url is None:
                        logger.error("Verified error, hash or url in request proof is None")
                    elif url != download_url:
                        logger.error("Verified error, the url is not matched")
                    elif vsw.utils.generate_digest(download_url) != credential_hash:
                        logger.error("Verified error, the digest is not matched")
                    else:
                        logger.info('Congratulation! verified successfully!')
                else:
                    logger.error("Verified error, the name in presentation request is wrong")
                break;
            else:
                times += 1

        if times > timeout:
            logger.error("verified error, presentation proof result is not verified!")


def retrieve_result(presentation_exchange_id):
    time.sleep(1)  # wait communicate complete automatically between agents
    presentation_proof_result = get_vsw_proof(presentation_exchange_id)
    return presentation_proof_result


def get_vsw_proof(pres_ex_id):
    vsw_url = urljoin(vsw_url_host, f"/present-proof/records/{pres_ex_id}")
    res = requests.get(vsw_url)
    return json.loads(res.text)


def check_credential(issuer_did, software_name, download_url):
    wql = json.dumps({"attr::developer-did::value": issuer_did, "attr::software-name::value": software_name,
                      "schema_name": "software-certificate", "attr::url::value": download_url})
    repo_url = f"{repo_url_host}/credentials?wql={wql}"
    res = requests.get(repo_url)
    return json.loads(res.text)["results"]


def send_request(client_conn_id, software_name, issuer_did):
    """
    Verifier sends request (prover receives request)
    Request Proof (from verifier to prover, required)
    """
    vsw_url = f'{vsw_url_host}/present-proof/send-request'
    req_attrs = [
        {
            "name": "software-name",
            "restrictions": [{"schema_name": "software-certificate", "attr::developer-did::value": issuer_did, "attr::software-name::value": software_name}]
        },
        {
            "name": "software-version",
            "restrictions": [{"schema_name": "software-certificate", "attr::developer-did::value": issuer_did, "attr::software-name::value": software_name}]
        },
        {
            "name": "developer-did",
            "restrictions": [{"schema_name": "software-certificate", "attr::developer-did::value": issuer_did, "attr::software-name::value": software_name}]
        },
        {
            "name": "hash",
            "restrictions": [{"schema_name": "software-certificate", "attr::developer-did::value": issuer_did, "attr::software-name::value": software_name}]
        },
        {
            "name": "url",
            "restrictions": [{"schema_name": "software-certificate", "attr::developer-did::value": issuer_did, "attr::software-name::value": software_name}]
        }
    ]

    indy_proof_request = {
        "name": "Proof of Software Certificate",
        "version": "1.0",
        "nonce": str(uuid4().int),
        "requested_attributes": {
            f"0_{req_attr['name']}_uuid": req_attr
            for req_attr in req_attrs
        },
        "requested_predicates": {}
    }
    proof_request_web_request = {
        "connection_id": client_conn_id,
        "proof_request": indy_proof_request
    }
    res = requests.post(vsw_url, json=proof_request_web_request)
    return json.loads(res.text)


def get_client_connection():
    connection_response = requests.get(f'{vsw_url_host}/connections')
    res = json.loads(connection_response.text)
    connections = res["results"]
    return connections[-1]



from vsw.commands.list import get_connections
from vsw.commands.list import get_wallet
from vsw.commands.list import get_schema
from vsw.commands.list import get_credential_definition
from vsw.utils import get_vsw_agent


def test_get_connections():
    vsw_config = get_vsw_agent()
    get_connections(vsw_config)


def test_get_wallet():
    vsw_config = get_vsw_agent()
    get_wallet(vsw_config)


def test_get_schema():
    vsw_config = get_vsw_agent()
    get_schema(vsw_config)


def test_get_credential_definition():
    vsw_config = get_vsw_agent()
    get_credential_definition(vsw_config)


import argparse
import json
import time
from typing import List
from vsw.log import Log
from urllib.parse import urljoin
import vsw.utils
import requests
import validators

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    try:
        args = parse_args(args)
        software_name = input('Please enter software name: ')
        software_version = input('Please enter software version: ')
        software_did = input('Please enter software did: ')
        software_url = input('Please enter software package url: ')
        software_alt_url1 = args.alt_url1
        software_alt_url2 = args.alt_url2
        if software_url and not validators.url(software_url):
            print('The software package url is wrong, please check')
            return
        if software_alt_url1 and not validators.url(software_alt_url1):
            print('The software package alt-url1 is wrong, please check')
            return
        if software_alt_url2 and not validators.url(software_alt_url2):
            print('The software package alt-url2 is wrong, please check')
            return
        issue_credential(software_name, software_version, software_did, software_url, software_alt_url1, software_alt_url2)
    except KeyboardInterrupt:
        print(" ==> Exit publish!")


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("--alt-url1", required=False, help="The software optional package url1")
    parser.add_argument("--alt-url2", required=False, help="The software optional package url2")
    return parser.parse_args(args)


def issue_credential(software_name, software_version, software_did, software_url, software_alt_url1, software_alt_url2):
    logger.info("executing publish, please waiting for response")
    connection = get_repo_connection()
    proposal_response = send_proposal(connection["connection_id"], get_public_did(), software_name,
                                      software_version, software_did, software_url, software_alt_url1,
                                      software_alt_url2)
    credential_exchange_id = proposal_response["credential_exchange_id"]
    logger.info(f'credential_exchange_id: {credential_exchange_id}')

    times = 0
    while times <= timeout:
        res = retrieve_result(credential_exchange_id)
        print(f'waiting state update, current state: {res["state"]}')
        if res["state"] == "credential_acked":
            logger.info("Congratulation, execute publish successfully!")
            break;
        else:
            times += 1;
    if times > timeout:
        logger.error("Sorry, there might be some issue during publishing")


def retrieve_result(credential_exchange_id):
    time.sleep(1)  # wait communicate complete automatically between agents
    res = get_credential_record(credential_exchange_id)
    return res


def get_repo_connection():
    connection_response = requests.get(f'{repo_url_host}/connections')
    res = json.loads(connection_response.text)
    connections = res["results"]
    return connections[-1]


def get_public_did():
    url = urljoin(vsw_url_host, "/wallet/did/public")
    response = requests.get(url)
    res = json.loads(response.text)
    return res["result"]["did"]


def get_credential_record(cred_ex_id):
    url = urljoin(repo_url_host, f"/issue-credential/records/{cred_ex_id}")
    credential_response = requests.get(url)
    res = json.loads(credential_response.text)
    return res


def send_proposal(repo_conn_id, developer_did, software_name, software_version, software_did, software_url,
                  software_alt_url1, software_alt_url2):
    if software_alt_url1 is None:
        software_alt_url1 = ""
    if software_alt_url2 is None:
        software_alt_url2 = ""
    digest = vsw.utils.generate_digest(software_url)
    vsw_repo_url = f'{repo_url_host}/issue-credential/send-proposal'
    res = requests.post(vsw_repo_url, json={
        "comment": "execute vsw publish cli",
        "auto_remove": False,
        "trace": True,
        "connection_id": repo_conn_id,
        "credential_proposal": {
            "@type": f"did:sov:{developer_did};spec/issue-credential/1.0/credential-preview",
            "attributes": [
                {
                    "name": "developer-did",
                    "value": developer_did
                },
                {
                    "name": "software-version",
                    "value": software_version
                },
                {
                    "name": "software-name",
                    "value": software_name
                },
                {
                    "name": "software-did",
                    "value": software_did
                },
                {
                    "name": "url",
                    "value": software_url
                },
                {
                    "name": "alt-url1",
                    "value": software_alt_url1
                },
                {
                    "name": "alt-url2",
                    "value": software_alt_url2
                },
                {
                    "name": "hash",
                    "value": digest
                },
                {
                    "name": "alt-hash",
                    "value": ""
                }
            ]
        },
    })
    return json.loads(res.text)



from vsw.commands import verify


def test_main():
    verify.execute("FF", "2mKdrR6QpmqNp8eHnkCogV", "http://images.pccoo.cn/bar/2012426/20124261343081s.jpg")


def test_retrieve_results():
    verify.retrieve_result("01168b93-ea56-452e-b7ca-e14d4be1bfca")


def test_check_credential():
    verify.check_credential("1234", "python")


import os
import socket
import subprocess
from typing import List

from vsw.log import Log
from vsw.utils import get_vsw_agent

logger = Log(__name__).logger


def main(args: List[str]) -> bool:
    kill_lt()
    kill_vsw()
    print("Exited vsw")


def kill_vsw():
    configuration = get_vsw_agent()
    a_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    location = (configuration.get("admin_host"), int(configuration.get("admin_port")))
    result_of_check = a_socket.connect_ex(location)
    if result_of_check == 0:
        subprocess.Popen(f'kill -9 $(lsof -t -i:{configuration.get("admin_port")})', stdout=subprocess.DEVNULL, shell=True)


def kill_lt():
    out = os.popen("ps aux | grep npx").read()
    for line in out.splitlines():
        if 'localtunnel' in line:
            pid = int(line.split()[1])
            subprocess.run(f'kill -9 {pid}', stdout=subprocess.DEVNULL, shell=True)



from setuptools import setup

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    install_requires=requirements,
    package_data={'vsw.conf':['*.ini','*.txt','*.sh']}
)
